#include "tstring.h"

// the funny thing is that the function only differs in typecast
tstring &tstring::assign(tstring &_tstr)
{
   return assign((TCHAR *)_tstr.c_str());
}

tstring &tstring::assign(const tstring &_tstr)
{
   return assign((TCHAR *)_tstr.c_str());
}

// hopefully the std::copy command will work just fine because the wchar_t shouldn't be different
// between tsring and the native type of string
tstring &tstring::assign(string &_str)
{
   return assign((char *)_str.c_str());
}

tstring &tstring::assign(wstring &_wstr)
{
   return assign((wchar_t *)_wstr.c_str());
}

// we need to replace all assign commands. Having the wchar_t and char handled will make it easier
// since all string functions have to extract the .c_str() anyways and put the info into the tstring
// manually
tstring &tstring::assign(wchar_t *wc_str)
{
   this->resize(wcslen(wc_str) + 1);
#ifdef _UNICODE
   for ( size_t x = 0; x < (wcslen(wc_str) + 1); x++ )
   {
      this->at(x) = wc_str[x];
   }
#else
   // basically the same as above, but first convert to multibyte
   size_t charsConverted;

   char *dest = new char[wcslen(wc_str) + 1];
   wcstombs_s(&charsConverted, dest, wcslen(wc_str) + 1, wc_str, wcslen(wc_str) + 1);
   
   for ( size_t x = 0; x < (wcslen(wc_str) + 1); x++ )
   {
      this->at(x) = dest[x];
   }

   delete dest;
#endif

   return *this;
}

tstring &tstring::assign(char *c_str)
{
   this->resize(strlen(c_str) + 1);

#ifdef _UNICODE
   TCHAR *wctemp = new TCHAR[strlen(c_str) + 1];

   if ( wctemp != NULL )
   {
      size_t charsConverted;

      mbstowcs_s(&charsConverted, wctemp, strlen(c_str) + 1, c_str, strlen(c_str) + 1);

      for ( size_t x = 0; x < (wcslen(wctemp) + 1); x++ )
      {
         this->at(x) = wctemp[x];
      }

      delete wctemp;
   }
#else
   for ( size_t x = 0; x < (strlen(c_str) + 1); x++ )
   {
      this->at(x) = c_str[x];
   }
#endif
   return *this;
}


tstring &tstring::operator=(const tstring &rhs)
{
   if ( this != &rhs )
   {
      assign(rhs);
   }

   return *this;
}

tstring &tstring::operator=(tstring &rhs)
{
   if ( this != &rhs )
   {
      assign(rhs);
   }

   return *this;
}

/*
tstring &tstring::operator=(tstring &rhs)
{
   if ( this != &rhs )
   {
      assign(rhs);
   }

   return *this;
}*/

tstring &tstring::operator=(string &rhs)
{
#ifdef _UNICODE
   assign(rhs);
#else
   if ( this != &rhs )
   {
      assign(rhs);
   }
#endif

   return *this;
}

tstring &tstring::operator=(wstring &rhs)
{
#ifdef _UNICODE
   if ( this != &rhs )
   {
      assign(rhs);
   }
#else
   assign(rhs);
#endif

   return *this;
}


tstring &tstring::operator=(char *rhs)
{
   return assign(rhs);
}

tstring &tstring::operator=(wchar_t *rhs)
{
   return assign(rhs);
}

// this needs to be done because there's no guarantee that there will be an output to stay 
// with the result after the function ends. I could have put 
void tstring::mbc_str(char *output, size_t maxlen)
{
#ifdef _UNICODE
   size_t charsConverted;
   wcstombs_s(&charsConverted, output, this->size(), this->c_str(), maxlen);
#else
   strcpy_s(output, maxlen, this->c_str());
#endif
}

void tstring::wc_str(wchar_t *output, size_t maxlen)
{
#ifdef _UNICODE
   wcscpy_s(output, maxlen, this->c_str());
#else
   size_t charsConverted;
   mbstowcs_s(&charsConverted, output, this->size(), this->c_str(), maxlen);
#endif
}


// multibyte c++ string. 
void tstring::mbcpp_str(string *out)
{
#ifdef UNICODE
   size_t charsConverted;
   char *in = new char[this->size() + 1];
   
   wcstombs_s(&charsConverted, in, this->size(), this->c_str(), this->size());
   
   out->assign(in);

   delete in;
#else
   out->assign(this->c_str());
#endif
}

// Wide c++ string
void tstring::wcpp_str(wstring *out)
{
#ifdef UNICODE
   out->assign(this->c_str());
#else

   size_t charsConverted;
   wchar_t *in = new wchar_t[this->size() + 1];
   
   mbstowcs_s(&charsConverted, in, this->size(), this->c_str(), this->size());
   
   out->assign(in);

   delete in;
#endif
}

// append two tstrings together. Might have to revist in case this damages something
//
// Mostly fixes the problem where the program put a phantom space at the end
tstring& tstring::append(const tstring &_tstr)
{
   size_t oldsize = this->size() - 1;
   this->resize( oldsize + _tstr.size() - 1);
   
   size_t s, c = 0;
   for ( s = oldsize; s < this->size(); s++ )
   {
      this->at(s) = _tstr.at(c);
      c++;
   }

   return *this;
}

// append a TCHAR * to the string
tstring& tstring::append(const TCHAR *tc)
{
   size_t oldSize = this->size() - 1;
   size_t newSize = _tcslen(tc);

   this->resize(oldSize + newSize);

   size_t c = 0;
   for ( size_t s = oldSize; s < this->size(); s++ )
   {
      this->at(s) = tc[c];
      c++;
   }

   return *this;
}

// adds a string at the end
tstring& tstring::operator+=(const tstring &rhs)
{
   this->append(rhs);

   return *this;
}

tstring& tstring::operator+=(const TCHAR *rhs)
{
   this->append(rhs);

   return *this;
}

// operator+, adds two tstrings together
tstring tstring::operator+(const tstring &rhs)
{
   tstring result;

   result = *this; 
   result.append(rhs);

   return result;
}

tstring tstring::operator+(const TCHAR   *rhs)
{
   tstring result;

   result = *this; 
   result.append(rhs);

   return result;
}

// assigns an UTF-8 string. This is going to be tricky since a UTF-8 string ranges 
// anywhere between 1 byte to 4 bytes. However, it shouldn't be bad since the 0-127 characters
// are the same between ANSI and UTF-8, so strlen should work just fine
//
// Once this is done, I should be able to use the vanilla assign function so that function should take
// care of the rest and any other conversions, like to cpp string format
tstring& tstring::assignUTF8(char *c_str)
{
   wchar_t *destWStr = new wchar_t[strlen(c_str)];

   // uses PRECOMPOSED by default
   MultiByteToWideChar(CP_UTF8, 0, c_str, strlen(c_str), destWStr, strlen(c_str));
   assign(destWStr);

   delete destWStr;
   return *this;
}

// assigns an UTF-8 string by using compostie conversion. 
tstring& tstring::assignUTF8Compostie(char *c_str)
{
   wchar_t *destWStr = new wchar_t[strlen(c_str)];

   MultiByteToWideChar(CP_UTF8, MB_COMPOSITE, c_str, strlen(c_str), destWStr, strlen(c_str));
   assign(destWStr);

   delete destWStr;
   return *this;
}

// operators == and !=. The TCHAR version needed fixing while the tstring version didn't
bool tstring::operator==(const TCHAR *rhs)
{
   tstring arHS;
   arHS.assign((TCHAR *)rhs);

   return (*this) == arHS;
}

bool tstring::operator==(const tstring &rhs)
{
   if ( this->size() != rhs.size() )
      return false;

   for ( size_t s = 0; s < rhs.size(); s++ )
   {
      if ( this->at(s) != rhs.at(s) ) 
         return false;
   }

   return true;
}

// mostly I copied operator== and reverse the true and false
bool tstring::operator!=(const TCHAR *rhs)
{
   return !( (*this) == rhs);
}

bool tstring::operator!=(const tstring &rhs)
{
   return !( (*this) == rhs);
}
